#! /usr/bin/env python

# Copyright (C) 2007 Daniel Watkins
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import subprocess
import tempfile
import os

document_template = r"""\documentclass[english]{article}
\usepackage[T1]{fontenc}
\usepackage[latin1]{inputenc}

\pagestyle{empty}
\makeatletter
\usepackage{babel}
\usepackage{amssymb}
\usepackage{amsmath}
\makeatother
\begin{document}
%s
\end{document}"""


class LatexObject(object):

    def __init__(self, formula):
        self.object = formula
        self.tex = document_template % (self.object,)
        self.temp_dir = None

    def finished(self):
        if self.temp_dir is not None:
            for root, dirs, files in os.walk(self.temp_dir, topdown=False):
                for name in files:
                    os.remove(os.path.join(root, name))
                for name in dirs:
                    os.rmdir(os.path.join(root, name))
            os.rmdir(self.temp_dir)
            self.temp_dir = None

    def get_tex_string(self):
        """Returns a TeX file that will generate a document containing only the
        formula represented by this object."""

        return self.tex

    def generate_dvi(self):
        if self.temp_dir is None:
            self.temp_dir = tempfile.mkdtemp()

        self.generate_latex()

        os.chdir(self.temp_dir)
        subprocess.Popen(
            ["latex", "-interaction", "batchmode", "formula.tex"],
            stdout=subprocess.PIPE, stderr=subprocess.PIPE).wait()

        return "%s/formula.dvi" % (self.temp_dir,)

    def generate_latex(self):
        if self.temp_dir is None:
            self.temp_dir = tempfile.mkdtemp()

        filename = "%s/formula.tex" % (self.temp_dir,)

        latex_file = file(filename, "w")
        latex_file.write(self.tex)
        latex_file.close()

        return filename

    def generate_png(self):
        if self.temp_dir is None:
            self.temp_dir = tempfile.mkdtemp()

        self.generate_dvi()

        os.chdir(self.temp_dir)
        subprocess.Popen(
            ["dvipng", "-x", "2986", "-T", "tight", "formula.dvi", "-o", "formula.png"],
            stdout=subprocess.PIPE,stderr=subprocess.PIPE).wait()

        return "%s/formula.png" % (self.temp_dir,)


class LatexFormula(LatexObject):
    def __init__(self, formula):
        self.object = "$$%s$$" % (formula.replace("$", ""))
        self.tex = document_template % (self.object,)
        self.temp_dir = None
