import unittest
import latex
import tempfile
import os.path
import difflib

class TestCase(unittest.TestCase):

    def _ndiff_strings(self, a, b):
        """Return ndiff between two strings containing lines.

        A trailing newline is added if missing to make the strings
        print properly."""
        if b and b[-1] != '\n':
            b += '\n'
        if a and a[-1] != '\n':
            a += '\n'
        difflines = difflib.ndiff(a.splitlines(True),
                                  b.splitlines(True),
                                  linejunk=lambda x: False,
                                  charjunk=lambda x: False)
        return ''.join(difflines)


    def assertEqualDiff(self, a, b, message=None):
        """Assert two texts are equal, if not raise an exception.

        This is intended for use with multi-line strings where it can 
        be hard to find the differences by eye.
        """
        # TODO: perhaps override assertEquals to call this for strings?
        if a == b:
            return
        if message is None:
            message = "texts not equal:\n"
        raise AssertionError(message +
                             self._ndiff_strings(a, b))

class TestFormula(TestCase):
    def test_generate_tex_string(self):
        out = latex.LatexFormula(r"\frac{a}{b}").get_tex_string()
        expected = \
r"""\documentclass[english]{article}
\usepackage[T1]{fontenc}
\usepackage[latin1]{inputenc}

\pagestyle{empty}
\makeatletter
\usepackage{babel}
\usepackage{amssymb}
\usepackage{amsmath}
\makeatother
\begin{document}
$$\frac{a}{b}$$
\end{document}"""

        self.assertEqualDiff(out, expected)

class TestObject(TestCase):

    def test_generate_dvi(self):
        formula = latex.LatexObject(r"$$\frac{a}{b}$$")
        path = formula.generate_dvi()
        temp_dir = formula.temp_dir
        expected_path = "%s/formula.dvi" % (temp_dir,)

        self.assertEquals(path, expected_path)
        self.assert_(os.path.exists(expected_path))
        self.assert_(os.path.exists(path))

        formula.finished()

    def test_write_latex_file(self):
        formula = latex.LatexObject(r"$$\frac{a}{b}$$")
        path = formula.generate_latex()
        temp_dir = formula.temp_dir
        expected_path = "%s/formula.tex" % (temp_dir,)

        self.assertEquals(path, expected_path)
        self.assert_(os.path.exists(expected_path))
        self.assert_(os.path.exists(path))

        formula.finished()

    def test_generate_png(self):
        formula = latex.LatexObject(r"$$\frac{a}{b}$$")
        path = formula.generate_png()
        temp_dir = formula.temp_dir
        expected_path = "%s/formula.png" % (temp_dir,)

        self.assertEquals(path, expected_path)
        self.assert_(os.path.exists(expected_path))
        self.assert_(os.path.exists(path))

        formula.finished()

    def test_cleanup(self):
        formula = latex.LatexObject(r"$$\frac{a}{b}$$")
        formula.generate_latex()

        temp_dir = formula.temp_dir
        self.assert_(os.path.exists(temp_dir))

        formula.finished()
        self.assert_(not os.path.exists(temp_dir))

    def test_cleanup_twice(self):
        formula = latex.LatexObject(r"$$\frac{a}{b}$$")
        formula.generate_latex()

        temp_dir = formula.temp_dir
        self.assert_(os.path.exists(temp_dir))

        formula.finished()
        self.assert_(not os.path.exists(temp_dir))
        formula.finished()
        self.assert_(not os.path.exists(temp_dir))


if __name__ == '__main__':
    unittest.main()
